/**
 *
 * Magnolia controls module (http://www.openmindlab.com/lab/products/controls.html)
 * Copyright (C)2008 - 2010, Openmind S.r.l. http://www.openmindonline.it
 *
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

package net.sourceforge.openutils.mgnlcontrols.dialog;

import info.magnolia.cms.beans.config.ContentRepository;
import info.magnolia.cms.core.Content;
import info.magnolia.cms.core.ItemType;
import info.magnolia.cms.util.ContentUtil;
import info.magnolia.cms.util.NodeDataUtil;

import java.util.ArrayList;
import java.util.List;
import java.util.Map;

import javax.jcr.RepositoryException;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.apache.commons.lang.StringUtils;


/**
 * @author fgiust
 * @version $Id: DialogGrid.java 2460 2010-05-17 09:51:33Z cstrap $
 */
public class DialogGrid extends ConfigurableFreemarkerDialog
{

    private List<Content> colConfigs = new ArrayList<Content>();

    /**
     * {@inheritDoc}
     */
    @Override
    public void init(HttpServletRequest request, HttpServletResponse response, Content websiteNode, Content configNode)
        throws RepositoryException
    {
        super.init(request, response, websiteNode, configNode);
        if (StringUtils.isEmpty(getConfigValue("saveHandler")))
        {
            setConfig("saveHandler", "net.sourceforge.openutils.mgnlcontrols.dialog.DialogGridSaveHandler");
        }
        if (configNode != null && configNode.hasContent("columns"))
        {
            colConfigs.addAll(configNode.getContent("columns").getChildren(ItemType.CONTENTNODE));
        }
    }

    /**
     * {@inheritDoc}
     */
    @Override
    protected String getPath()
    {
        return "dialogs/grid.ftl";
    }

    /**
     * {@inheritDoc}
     */
    @Override
    protected void addToParameters(Map<String, Object> parameters)
    {
        super.addToParameters(parameters);
        String value = getValue();
        if (value != null)
        {
            StringBuffer sbValue = new StringBuffer(value.length());
            StringBuffer sbLine = new StringBuffer();
            for (String line : StringUtils.splitPreserveAllTokens(value, '\n'))
            {
                line = StringUtils.removeEnd(line, "\r");
                int colIndex = 0;
                for (String token : StringUtils.splitPreserveAllTokens(line, '\t'))
                {
                    if (colIndex < colConfigs.size())
                    {
                        Content colConfig = colConfigs.get(colIndex);
                        String colType = NodeDataUtil.getString(colConfig, "type");
                        if ("uuidLink".equals(colType))
                        {
                            String repository = StringUtils.defaultIfEmpty(NodeDataUtil.getString(
                                colConfig,
                                "repository"), ContentRepository.WEBSITE);
                            Content node = ContentUtil.getContentByUUID(repository, token);
                            if (node != null)
                            {
                                token = node.getHandle();
                            }
                        }
                        else if ("media".equals(colType))
                        {
                            String valueType = StringUtils.defaultIfEmpty(NodeDataUtil
                                .getString(colConfig, "valueType"), "uuid");
                            if ("uuid".equals(valueType))
                            {
                                if (StringUtils.startsWith(token, "/"))
                                {
                                    Content node = ContentUtil.getContentByUUID("media", token);
                                    token = node != null ? node.getUUID() : token;
                                }
                            }
                        }
                    }
                    if (colIndex > 0)
                    {
                        sbLine.append('\t');
                    }
                    sbLine.append(token);
                    colIndex++;
                }
                if (sbValue.length() > 0)
                {
                    sbValue.append('\n');
                }
                sbValue.append(sbLine);
                sbLine.setLength(0);
            }
            value = new String(sbValue);
        }
        parameters.put("gridValue", value);
    }

}
