/**
 *
 * Magnolia controls module (http://www.openmindlab.com/lab/products/controls.html)
 * Copyright (C)2008 - 2010, Openmind S.r.l. http://www.openmindonline.it
 *
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

package net.sourceforge.openutils.mgnlcontrols.dialog;

import info.magnolia.cms.beans.config.ContentRepository;
import info.magnolia.cms.beans.runtime.Document;
import info.magnolia.cms.beans.runtime.MultipartForm;
import info.magnolia.cms.core.Content;
import info.magnolia.cms.core.HierarchyManager;
import info.magnolia.cms.core.ItemType;
import info.magnolia.cms.core.NodeData;
import info.magnolia.cms.core.Path;
import info.magnolia.cms.gui.fckeditor.FCKEditorTmpFiles;
import info.magnolia.cms.security.AccessDeniedException;
import info.magnolia.cms.util.ContentUtil;
import info.magnolia.cms.util.NodeDataUtil;
import info.magnolia.context.MgnlContext;
import info.magnolia.module.admininterface.FieldSaveHandler;
import info.magnolia.module.admininterface.SaveHandlerImpl;

import java.io.IOException;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;

import javax.jcr.RepositoryException;

import org.apache.commons.io.FileUtils;
import org.apache.commons.lang.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;


/**
 * Needed by file upload. Setup: assign this class name as a "saveHandler" property of the grid field's configuration
 * node.
 * @author dschivo
 */
public class DialogGridSaveHandler implements FieldSaveHandler
{

    private static Logger log = LoggerFactory.getLogger(DialogGridSaveHandler.class);

    /**
     * {@inheritDoc}
     */
    public void save(Content parentNode, Content configNode, String name, MultipartForm form, int type, int valueType,
        int isRichEditValue, int encoding) throws RepositoryException, AccessDeniedException
    {
        List<Content> colConfigs = new ArrayList<Content>();
        if (configNode != null && configNode.hasContent("columns"))
        {
            colConfigs.addAll(configNode.getContent("columns").getChildren(ItemType.CONTENTNODE));
        }

        HierarchyManager hm = parentNode.getHierarchyManager();
        Content filesNode = ContentUtil.getOrCreateContent(parentNode, name + "_files", ItemType.CONTENTNODE);
        String value = form.getParameter(name);
        String ctx = MgnlContext.getContextPath();
        String tmpPath = "/tmp/fckeditor/";
        List<String> usedFiles = new ArrayList<String>();
        StringBuffer sbValue = new StringBuffer(value.length());
        StringBuffer sbLine = new StringBuffer();
        for (String line : StringUtils.splitPreserveAllTokens(value, '\n'))
        {
            line = StringUtils.removeEnd(line, "\r");
            int colIndex = 0;
            for (String token : StringUtils.splitPreserveAllTokens(line, '\t'))
            {
                if (colIndex < colConfigs.size())
                {
                    Content colConfig = colConfigs.get(colIndex);
                    String colType = NodeDataUtil.getString(colConfig, "type");
                    if ("uuidLink".equals(colType))
                    {
                        String repository = StringUtils.defaultIfEmpty(
                            NodeDataUtil.getString(colConfig, "repository"),
                            ContentRepository.WEBSITE);
                        Content node = ContentUtil.getContent(repository, token);
                        if (node != null)
                        {
                            token = node.getUUID();
                        }
                    }
                    else if ("media".equals(colType))
                    {
                        String mediaValueType = StringUtils.defaultIfEmpty(NodeDataUtil.getString(
                            colConfig,
                            "valueType"), "uuid");
                        if ("uuid".equals(mediaValueType))
                        {
                            if (StringUtils.startsWith(token, "/"))
                            {
                                Content node = ContentUtil.getContentByUUID("media", token);
                                token = node != null ? node.getUUID() : token;
                            }
                        }
                    }
                    else if ("file".equals(colType))
                    {
                        if (token.startsWith(ctx))
                        {
                            String link = StringUtils.substringAfter(token, ctx);
                            if (StringUtils.startsWith(link, tmpPath))
                            {
                                String uuid = StringUtils.substringBetween(link, tmpPath, "/");
                                Document doc = FCKEditorTmpFiles.getDocument(uuid);
                                String fileNodeName = Path.getUniqueLabel(hm, filesNode.getHandle(), "file");
                                SaveHandlerImpl.saveDocument(filesNode, doc, fileNodeName, "", "");
                                link = filesNode.getHandle()
                                    + "/"
                                    + fileNodeName
                                    + "/"
                                    + doc.getFileNameWithExtension();
                                token = link;
                                doc.delete();
                                try
                                {
                                    FileUtils.deleteDirectory(new java.io.File(Path.getTempDirectory()
                                        + "/fckeditor/"
                                        + uuid));
                                }
                                catch (IOException e)
                                {
                                    log.error("can't delete tmp file ["
                                        + Path.getTempDirectory()
                                        + "/fckeditor/"
                                        + uuid
                                        + "]");
                                }
                            }
                        }
                        if (token.startsWith(filesNode.getHandle()))
                        {
                            String fileNodeName = StringUtils.removeStart(token, filesNode.getHandle() + "/");
                            fileNodeName = StringUtils.substringBefore(fileNodeName, "/");
                            usedFiles.add(fileNodeName);
                        }
                    }
                }
                if (colIndex > 0)
                {
                    sbLine.append('\t');
                }
                sbLine.append(token);
                colIndex++;
            }
            if (sbValue.length() > 0)
            {
                sbValue.append('\n');
            }
            sbValue.append(sbLine);
            sbLine.setLength(0);
        }
        for (Iterator<NodeData> iter = filesNode.getNodeDataCollection().iterator(); iter.hasNext();)
        {
            NodeData fileNodeData = iter.next();
            if (!usedFiles.contains(fileNodeData.getName()))
            {
                fileNodeData.delete();
            }
        }
        NodeDataUtil.getOrCreateAndSet(parentNode, name, new String(sbValue));
    }

}
